// Etape 1 :
// Importation des packages Java 2
import java.applet.Applet;
import java.awt.*;

// Etape 2 :
// Importation des packages Java 3D
import com.sun.j3d.utils.applet.MainFrame;
import com.sun.j3d.utils.universe.*;
import com.sun.j3d.utils.geometry.*;
import javax.media.j3d.*;
import javax.vecmath.*;

public class TranslationRotation extends Applet {

  public TranslationRotation() {
    this.setLayout(new BorderLayout());

    // Etape 3 :
    // Creation du Canvas 3D
    Canvas3D canvas3D = new Canvas3D(SimpleUniverse.getPreferredConfiguration());

    this.add(canvas3D, BorderLayout.CENTER);

    // Etape 4 :
    // Creation d'un objet SimpleUniverse
    SimpleUniverse simpleU = new SimpleUniverse(canvas3D);

    // Etape 5 :
    // Positionnement du point d'observation pour avoir une vue correcte de la
    // scene 3D
    simpleU.getViewingPlatform().setNominalViewingTransform();

    // Etape 6 :
    // Creation de la scene 3D qui contient tous les objets 3D que l'on veut
    // visualiser
    BranchGroup scene = createSceneGraph();

    // Etape 7 :
    // Compilation de la scene 3D
    scene.compile();

    // Etape 8:
    // Attachement de la scene 3D a l'objet SimpleUniverse
    simpleU.addBranchGraph(scene);
  }

  /**
   * Creation de la scene 3D
   * @return scene 3D
   */
  public BranchGroup createSceneGraph() {
    // Creation de l'objet racine (parent) qui contiendra tous les autres
    // noeuds
    BranchGroup parent = new BranchGroup();

    // Creation de la premiere transformation (translation)
    Transform3D translation = new Transform3D();
    translation.setTranslation(new Vector3f(0.5f, 0.5f, 0f));
    TransformGroup transformGroup = new TransformGroup(translation);

    /**
     * Creation de la seconde transformation (rotation de 45 degres autour
     * d'un axe parallele a Z mais passant par le centre du cube)
     */
    // On recupere la transformation globale courante
    // (ici une simple translation)
    Transform3D currentTransform = new Transform3D();
    transformGroup.getTransform(currentTransform);

    // On sauvegarde les composantes de la translation associee a cette
    // transformation dans un vecteur
    Vector3f translationComponent = new Vector3f();
    currentTransform.get(translationComponent);

    // On annule les composantes de la translation associee a cette
    // transformation globale : cela revient a recentrer notre cube
    // au centre du repere 3D
    currentTransform.setTranslation(new Vector3f(0, 0, 0));

    // On applique la rotation de 30 degres autour de l'axe Z a notre cube
    // qui est maintenant au centre du repere 3D
    Transform3D rotation = new Transform3D();
    rotation.rotZ(30f * Math.PI/180f);

    // Cette rotation est appliquee apres la transformation courante
    currentTransform.mul(rotation, currentTransform);

    // On repositionne ensuite le cube a sa position d'origine en
    // reintroduisant les composantes de la translation que nous avions
    // supprime
    currentTransform.setTranslation(translationComponent);

    // On applique la transformation globale a l'objet transformGroup
    transformGroup.setTransform(currentTransform);

    // Creation de l'objet pour lequel on va appliquer cette transformation
    // (translation + rotation)
    ColorCube cube = new ColorCube(0.2f);

    // Le cube est fils du transformGroup qui contient la transformation
    // globale
    transformGroup.addChild(cube);

    // L'objet transformGroup est le fils de l'objet racine (BranchGroup)
    parent.addChild(transformGroup);

    return parent;
  }

  /**
   * Etape 9 :
   * Methode main() nous permettant d'utiliser cette classe comme une applet
   * ou une application.
   * @param args arguments
   */
  public static void main(String[] args) {
    Frame frame = new MainFrame(new TranslationRotation(), 256, 256);
  }
}
